"""
module: slideshowfolder
created by: Johnpaul Burbank <jpburbank@tegus.ca>
Date: July 8, 2007
"""

#Zope / Z3
from zope.interface import implements, providedBy, alsoProvides
from Products.Five.browser import BrowserView

#Plone
from Products.CMFCore.utils import getToolByName

#ATCT
from Products.ATContentTypes.interface.image import IATImage
from Products.ATContentTypes.interface.folder import IATFolder

#Project
from interfaces import ISlideShowView, IFolderSlideShowView, ISlideShowFolder
from events import SlideShowSetting
from config import TRANSITIONS

try:
    #For Zope 2.10.4
    from zope.annotation.interfaces import IAttributeAnnotatable
except ImportError:
    #For Zope 2.9
    from zope.app.annotation.interfaces import IAttributeAnnotatable

from logging import info

class SlideShowFolderView(BrowserView):
    implements(ISlideShowView)

    def slideTransition(self):
        """Returns type of slide transition"""
        settings = SlideShowSetting(self.context)
        return settings.slideTransition

    def slideDuration(self):
        """Returns the time in second before the next slide transition"""
        settings = SlideShowSetting(self.context)
        return settings.slideDuration
        
    def panAmount(self):
        """Returns the amount of pan to use in a pan or combo transition"""
        settings = SlideShowSetting(self.context)
        return settings.panAmount

    def zoomAmount(self):
        """Returns the amount of zoom to use in a zoom or combo transition"""
        settings = SlideShowSetting(self.context)
        return settings.zoomAmount

    def transitionTime(self):
        """Returns the time in second for a slide transition to take place"""
        settings = SlideShowSetting(self.context)
        return settings.transitionTime
        
    def showHeight(self):
        """Returns the height of the slideshow"""
        settings = SlideShowSetting(self.context)
        return settings.showHeight
        
    def showWidth(self):
        """Returns the width of the slideshow"""
        settings = SlideShowSetting(self.context)
        return settings.showWidth
        
    def thumbnails(self):
        """Returns a boolean whether thumbnail mode has been chosen"""
        settings = SlideShowSetting(self.context)
        return settings.thumbnails
        
    def fast(self):
        """Returns a boolean whether arrow fast has been chosen"""
        settings = SlideShowSetting(self.context)
        return settings.fast
        
    def arrows(self):
        """Returns a boolean whether arrow mode has been chosen"""
        settings = SlideShowSetting(self.context)
        return settings.arrows
        

    def fields(self):
        """Returns the fields for this view"""
        trans = {}
        trans['label'] = 'Slide Transition'
        trans['fieldName'] = 'slideTransition'
        trans['description'] = 'Transition between slides'
        trans['value'] = self.slideTransition
        trans['type'] = 'select'
        trans['options'] = TRANSITIONS
        dur = {}
        dur['label'] = 'Slide Duration'
        dur['description'] = 'Time in seconds between slide transitions'
        dur['fieldName'] = 'slideDuration'
        dur['value'] = self.slideDuration
        dur['type'] = 'text'
        pan = {}
        pan['label'] = 'Pan Amount'
        pan['fieldName'] = 'panAmount'
        pan['description'] = 'Amount to pan images.  Only used for pan and combo transitions.  Enter a value from 0 to 100 or rand for a random amount. You must set width smaller than the width of the smallest image for panning to work.'
        pan['value'] = self.panAmount
        pan['type'] = 'text'
        zoom = {}
        zoom['label'] = 'Zoom Amount'
        zoom['fieldName'] = 'zoomAmount'
        zoom['description'] = 'Amount to zoom images.  Only used for zoom and combo transitions.  Enter a value from 0 to 100 or rand for a random amount. Note that zoom effects can look a bit weird on Windows/IE.'
        zoom['value'] = self.zoomAmount
        zoom['type'] = 'text'
        transTime = {}
        transTime['label'] = 'Transition Time'
        transTime['description'] = 'Time in seconds for a slide transition to take place.'
        transTime['fieldName'] = 'transitionTime'
        transTime['value'] = self.transitionTime
        transTime['type'] = 'text'
        width = {}
        width['label'] = 'Slideshow Width'
        width['description'] = 'Width of the slideshow. Defaults to the width of the first image.'
        width['fieldName'] = 'showWidth'
        width['value'] = self.showWidth
        width['type'] = 'text'
        height = {}
        height['label'] = 'Slideshow Height'
        height['description'] = 'Height of the slideshow. Defaults to the height of the first image.'
        height['fieldName'] = 'showHeight'
        height['value'] = self.showHeight
        height['type'] = 'text'
        thumb = {}
        thumb['label'] = 'Thumbnail Navigation'
        thumb['description'] = 'Include navigation thumbnails below the image.'
        thumb['fieldName'] = 'thumbnails'
        thumb['value'] = self.thumbnails
        thumb['type'] = 'checkbox'
        arrow = {}
        arrow['label'] = 'Arrow Navigation'
        arrow['description'] = 'Include forward and back arrows in navigation below image.'
        arrow['fieldName'] = 'arrows'
        arrow['value'] = self.arrows
        arrow['type'] = 'checkbox'
        fast = {}
        fast['label'] = 'Fast Mode'
        fast['description'] = 'Skips the transition when a new thumbnail is selected.'
        fast['fieldName'] = 'fast'
        fast['value'] = self.fast
        fast['type'] = 'checkbox'
        return (trans, dur, pan, zoom, transTime, width, height, thumb, arrow, fast)

    def getTransitionTypes(self):
        """Returns a tuple of possible slide transitions"""
        return TRANSITIONS
        
    def getFolderImageData(self):
        """Returns a list of image object belonging to the current slideshowfolder 
           and a string for the required javascript call, all in the form of a dictionary"""
        cat = getToolByName(self, 'portal_catalog')
        folder = self.context
        path = '/'.join(folder.getPhysicalPath())
        pathQuery = {'query' : path, 'depth' : 1 }

        results = cat(Type='Image', review_state='published', path=path, sort_on='getObjPositionInParent')
        
        if results:
            imageObj = results[0].getObject()
            imgHeight = imageObj.height
            imgWidth = imageObj.width
            image = '<img src="%s" width="%i" height="%i"/>' % (results[0].getURL(), imgWidth, imgHeight)
            names = ["%s/image_large" % item.id for item in results]
            imageDescriptions = [item.Description for item in results]
            
            #Raw data from settings
            slideTransitionSetting = self.slideTransition()
            slideDurationSetting = self.slideDuration()
            transitionPanAmount = self.panAmount()
            transitionZoomAmount = self.zoomAmount()
            transitionTimeSetting = self.transitionTime()
            showHeightSetting = self.showHeight()
            showWidthSetting = self.showWidth()
            thumbnailsSetting = self.thumbnails()
            arrowsSetting = self.arrows()
            fastSetting = self.fast()
            
            #Building the javascript string
            options = []
            transition = "type: '%s'" % slideTransitionSetting
            options.append(transition)
            pan = "pan: '%s'" % transitionPanAmount
            options.append(pan)
            zoom = "zoom: '%s'" % transitionZoomAmount
            options.append(zoom)
            if slideDurationSetting or transitionTimeSetting:
                duration = 'duration: [%i, %i]' % (int(transitionTimeSetting)*1000, int(slideDurationSetting)*1000)
                options.append(duration)
            if showHeightSetting <= 0:
                showHeightSetting = imgHeight
            height = 'height: %i' % showHeightSetting
            options.append(height)
            if showWidthSetting <= 0:
                showWidthSetting = imgWidth
            width = 'width: %i' % showWidthSetting
            options.append(width)
            if thumbnailsSetting:
                if arrowsSetting:
                    mode = "navigation: 'arrows+thumbnails'"
                    if fastSetting:
                        mode = "navigation: 'arrows+thumbnails fast'"
                elif fastSetting:
                    mode = "navigation: 'thumbnails fast'"
                else:  
                    mode = "navigation: 'thumbnails'"
            elif arrowsSetting:
                if fastSetting:
                    mode = "navigation: 'arrows fast'"
                else:
                    mode = "navigation: 'arrows'"
            if arrowsSetting or thumbnailsSetting:
                options.append(mode)
            if imageDescriptions:
                captions = 'captions: %s' % (str(imageDescriptions))
                options.append(captions)
            options.append('images: %s' % str(names))
                
            #Putting the information together
            showSettings = ', '.join(options)
            script = """
                     <script type="text/javascript">
                        myShow = new Slideshow('slideshow', {hu: '', %s});
                     </script>""" % (showSettings)
    
            data = {}
            data['image'] = image
            data['script'] = script
            return data
        return None


class FolderSlideShowView(BrowserView):
    """Provides view methods in regards to the transition from folder to slideshow folder"""
    implements(IFolderSlideShowView)
    iprovides = (IAttributeAnnotatable, ISlideShowFolder)

    def canSlideshow(self):
        """Determines if a folderish type can become a slideshowfolder"""
        contained = self.context.getChildNodes()
        if IATFolder.providedBy(self.context) and [node for node in contained if IATImage.providedBy(node)]:
            return True
        return False

    def makeSlideshow(self):
        """Make a folderish object a slideshowfolder"""
        for inter in self.iprovides:
            if not inter.providedBy(self.context):
                alsoProvides(self.context, inter)